%%%%%%%%%%%%%%%%%%%%%%%%%%
% https://www.iriscores.com
% vojta.ters@iriscores.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% GET DISPLAY RESOLUTION %%%%%%%
Displ = groot();
ScW = Displ.ScreenSize(3);
ScH = Displ.ScreenSize(4);
FiW = 1440;
FiH = 920;
Cls = perms([0,102,204]./255);
Cl_Brown      = 1;
Cl_Dark_Red   = 2;
Cl_Dark_Green = 3;
Cl_Violet     = 4;
Cl_Green      = 5;
Cl_Blue       = 6; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add release Directory to Matlab Path %
MexFilePath = ['..',filesep,'Cuda_FFT_Radix_248',filesep,'x64',filesep,'Release'];
addpath(MexFilePath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FFT Testing Configuration %
Radix = 2;                  % 2 / 4 / 8!
TW_Precalculate = 0;        % 0/1 - Precalculate Twiddles
USE_SharedMem = 1;          % 0/1 - Use Shared Memory (Up to Some Extend)
FFT_Max_Length = 1024*1024; % Maximum FFT Size Tested
Tol_Percent = 1.0;          % Error Toleration in respect to Matlab


FFT_Max_Power = log(FFT_Max_Length) / log(Radix);
for pw = 1:FFT_Max_Power
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Generate Random Data for Testing %
    Current_Length = Radix^(pw);
    DReal = single(randn(1,Current_Length));
    DImag = single(randn(1,Current_Length));
    DReal = DReal./max(DReal);
    DImag = DImag./max(DImag);
    Data = complex(DReal,DImag);
    disp(['Processing FFT Length: ',num2str(Current_Length)]);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Compute CUDA And Reference %
    Ref  = fft(Data);
    [Man,Indexes] = Cuda_FFT_Radix_248(Data,single([Radix,TW_Precalculate,USE_SharedMem]));
    RefIndexes = digitrevorder(0:Current_Length - 1,Radix);
    Equals = sum(Indexes == RefIndexes);
    if (Equals ~= Current_Length) 
        warning(['DigitReversal Failure: ',num2str(Equals),' / ',num2str(Current_Length),' Indexes Valid!']);
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check Reults for Problems %
    Tols = abs((Man - Ref)./Ref) * 100;
    OOR  = sum(Tols > Tol_Percent);
    Max_Rel = max(Tols);
    if OOR > 0
        warning(['Results out of Tolerance! for Length: ',num2str(Current_Length),' & Radix: ',num2str(Radix),' Err: ',num2str(Max_Rel),' %%', ' OutOfRanges: ',num2str(OOR)]);
    end
    
end

clear mex;